#ifndef SCIQLOPPLOTS_BINDINGS_H
#define SCIQLOPPLOTS_BINDINGS_H
#define QT_ANNOTATE_ACCESS_SPECIFIER(a) __attribute__((annotate(#a)))
#include <memory>
#include "SciQLopPlots/numpy_wrappers.hpp"

#include <qcustomplot.h>
#include <SciQLopPlots/SciQLopGraph.hpp>
#include "_QCustomPlot.hpp"

#endif // SCIQLOPPLOTS_BINDINGS_H
